function crm00050000_Comunicacao_Revenda_Controller(CRM_Utilities, crm00050000_Comunicacao_Revenda_Service, General){
    
    /**
     * Função para definir o tipo de 'save' que
     * será realizado (INSERT, UPDATE).
     * 
     * @param {object(widget)} widget widget
     * ativa onde será realizado o 'save'.
     * 
    */
    this.saveCommunition = function(widget){
    
        if(widget.currentRow.__is_new){
            
            this.saveNewCommunition(widget);
            
        }else{
            
            General.widgetSave(widget);
            
        }
        
    };
    
    /**
     * Função para inserir um novo registro na tablea
     * 'COMUNICAPARC' e salvar uma nova comunicação
     * associada a revenda em caso.
     * 
     * @param {object(widget)} widget widget
     * ativa onde está sendo realizado o cadastro
     * de uma nova comunicação.
     * 
    */
    this.saveNewCommunition = function(widget){
        
        var currentRow = widget.currentRow;
        
        this.insertNewPartnerCommunication(currentRow).then(function(response){
            
            var partnerCommunicationNumber = response.dataset.DADOS.NRCOMUNICAPARC;
            
            this.setPartnerCommunicationCode(currentRow, partnerCommunicationNumber);
            
            this.setColumnToReturnInCurrentRow(currentRow, "NRCOMUNICAREVENDA");
            
            General.widgetSave(widget).then(function(response){
            
                var communicationCode = response.dataset.columnsToReturn.NRCOMUNICAREVENDA;
                
                this.setCommunicationCodeAtCurrentRow(currentRow, communicationCode);
                
            }.bind(this));
            
        }.bind(this));
        
    };
    
    /**
     * Função para setar o número gerado para a comunicação
     * em sua currentRow.
     * 
     * @param {object} currentRow currentRow associada a
     * comunicação salva.
     * 
     * @param {int} communicationCode código gerado para
     * a comunicação.
    */
    this.setCommunicationCodeAtCurrentRow = function(currentRow, communicationCode){
        
        currentRow.NRCOMUNICAREVENDA = communicationCode;
        
    };
    
    /**
     * Função para setar a coluna a ser retornada
     * na resposta do 'save'.
     * 
     * @param {object} currentRow currentRow que receberá
     * a propriedade com o nome da coluna a ser retornada.
     * 
     * @param {string} columnName nome da coluna a ser
     * retornada.
    */
    this.setColumnToReturnInCurrentRow = function(currentRow, columnName){
    
        currentRow.__columns_to_return = [columnName];
        
    };
    
    /**
     * Função para setar o número de comunicação
     * do parceiro de negócio na currentRow recebida
     * como parâmetro.
     * 
     * @param {object} currentRow currentRow que irá 
     * receber o número de comunicação do parceiro.
     * 
     * @param {String} partnerCommunicationNumber
     * número de comunicação do parceiro que será
     * inserido na currentRow.
    */
    this.setPartnerCommunicationCode = function(currentRow, partnerCommunicationNumber){
        
        currentRow.NRCOMUNICAPARC = partnerCommunicationNumber;
        
    };
    
    /**
     * Função para definir os valores necessários
     * para um novo registro de comunicação na 
     * currentRow.
     * 
     * @param {object(widget)} widget widget ativa
     * no modo edição, onde está sendo realizado
     * um novo cadastro.
    */
    
    this.setValuesForNewCommunicationInCurrentRow = function(widget){
        
        this.setResaleNumberInCurrentRow(widget);
        
        this.setPartnerNumberInCurrentRow(widget);
        
        this.setLoggedOperatorInfos(widget);
        
    };
    
    /**
     * Função para receber o número da revenda 
     * em caso na currentRow do novo cadastro
     * de comunicação.
     * 
     * @param {object(widget)} widget widget ativa que
     * terá sua currentRow modificada.
    */
    this.setResaleNumberInCurrentRow = function(widget){
    
        widget.currentRow.NRREVENDA = widget.parent.parent.currentRow.NRREVENDA;
        
    };
    
    /**
     * Função para receber o número do parceiro 
     * da revenda em caso na currentRow do novo cadastro
     * de comunicação.
     * 
     * @param {object(widget)} widget widget ativa que
     * terá sua currentRow modificada.
    */
    this.setPartnerNumberInCurrentRow = function(widget){
        
        widget.currentRow.NRPARCNEGOCIO = widget.parent.parent.currentRow.NRPARCNEGOCIO;
        
    };
    
    /**
     * Função para setar as informações necessários
     * do operador logado na currentRow da widget
     * ativa.
     * 
     * @param {object(widget)} widget widget ativa
     * no ambiente de edição, que terá sua currentRow
     * modificada.
    */
    this.setLoggedOperatorInfos = function(widget){
        
        var currentRow = widget.currentRow;
        var parentWidget = widget.parent.parent;
        
        currentRow.NRORG = parentWidget.OPERATORLOGGED.NRORG;
        currentRow.CDOPERADOR = parentWidget.OPERATORLOGGED.CDOPERADOR;
            
    };
    
    /**
     * Função para inserir uma comunicação
     * ligado ao parceiro em caso na tabela
     * 'COMUNICAPARC'.
     * 
     * @param {obejct} currentRow currentRow 
     * onde se localiza as informações para o
     * novo registro.
    */
    this.insertNewPartnerCommunication = function(currentRow){
        
        return crm00050000_Comunicacao_Revenda_Service.callForInsertNewPartnerCommunication(currentRow);
        
    };
    
    /**
     * Função que executará todas as funções 
     * requisitadas no evento 'WidgetOnEnterOnce',
     * da widget de formas de comunicação.
     * 
     * @param {object} widget widget de formas de
     * comunicação.
    */
    this.resaleComunicationWidgetOnEnterOnce = function(widget){
        
        CRM_Utilities.setDefaultOrgFilterAndOperatorOrgFilter(widget.getField('NMFORMACOMU'));
        
        CRM_Utilities.setDefaultOrgNumberFilterValue(widget);
            
    };
    
}

Configuration(function(ContextRegister){
   ContextRegister.register("crm00050000_Comunicacao_Revenda_Controller", crm00050000_Comunicacao_Revenda_Controller); 
});
