/* global DeveloperToolsForm, inject, expect, Widget, Field, Container, Q, ZHPromise */
describe("DeveloperToolsForm", function(){

	var DevToolsForm;
	var DevToolsService;
	var templateManager;
	var $httpBackend;
	var $q;
	var $scope;
	var data = {
		"data": "value",
		"status": "404"
	};

	beforeEach( function(){
		window.templateUrl = "../../dyman/templates/dyman/";
		window.metadataUrl = "/json_test/test";
		window.serviceUrl = "../backend/service/index.php";
		module('ZeedhiApp');
		inject(function(_$httpBackend_, _metaDataFactory_, _ScreenService_, _templateManager_, _DevToolsService_, _$q_, _$rootScope_){
			DevToolsService = _DevToolsService_;
			templateManager = _templateManager_;
			DevToolsForm = new DeveloperToolsForm(_metaDataFactory_, _ScreenService_, templateManager, DevToolsService);
			$httpBackend = _$httpBackend_;
			$q = _$q_;
			$scope = _$rootScope_;
			$scope.$new();
		});
	});

	it("HasPropertyInJson should return true if the property's name exists on the json's object", function(){
		var json = {"name": "value"};
		var propertyName = "name";
		expect(DevToolsForm.hasPropertyInJson(json, propertyName)).toBe(true);
	});

	it("HasPropertyInJson should return false if the json's object entry is an array", function(){
		var json = ["id","value"];
		expect(DevToolsForm.hasPropertyInJson(json, 0)).toBe(false);
	});

	it("HasPropertyInJson should return false if the property's name doesn't exist on the json's object", function(){
		var json = {"name": "value"};
		var propertyName = "id";
		expect(DevToolsForm.hasPropertyInJson(json, propertyName)).toBe(false);
	});

	it("HasPropertyInJson should return false if the property's name is undefined or null", function(){
		var json = {"name": "value"};
		expect(DevToolsForm.hasPropertyInJson(json, undefined)).toBe(false);
		expect(DevToolsForm.hasPropertyInJson(json, null)).toBe(false);
	});

	it("IsCheckboxField should return true when the template has the string 'checkbox'", function(){
		var template = "checkbox";
		expect(DevToolsForm.isCheckboxField(template)).toBe(true);
	});

	it("IsCheckboxField should return false when the template doesn't have the string 'checkbox'", function(){
		var template = "itCan'tBeAnyStringValue";
		expect(DevToolsForm.isCheckboxField(template)).toBe(false);
	});

	it("GetFormWidgetAndContainer should return an object with widget and container", function(){
		var dependencies = {
			"dependsOn": {
				"anyProperty": "value"
			},
			"dependenciesTree": {
				"anyProperty": "value",
				"length":20
			}
		};
		var fieldsNames = ["anyProperty"];
		var docTemplate = {
			"$schema": "http://json-schema.org/draft-04/schema#",
			"description": "A form widget",
			"properties": {
				"anyProperty": {
					"type": "anyType",
					"description": "anyDescription"
				}
			},
			"required": ["anyProperty"],
			"title": "form.html",
			"type": "object"
		};
		var jsonItem = {"anyProperty": "anyValue"};
		var testObject = DevToolsForm.getFormWidgetAndContainer(fieldsNames, docTemplate, jsonItem, dependencies);
		expect(testObject.widget).toBeDefined();
		expect(testObject.container).toBeDefined();
	});

	it("GetFormWidgetAndContainer should return an object with widget and container when dependencies is empty", function(){
		var dependencies = {};
		var fieldsNames = ["anyProperty"];
		var docTemplate = {
			"$schema": "http://json-schema.org/draft-04/schema#",
			"description": "A form widget",
			"properties": {
				"anyProperty": {
					"type": "anyType",
					"description": "anyDescription"
				}
			},
			"required": ["anyProperty"],
			"title": "form.html",
			"type": "object"
		};
		var jsonItem = {"anyProperty": "anyValue"};
		var testObject = DevToolsForm.getFormWidgetAndContainer(fieldsNames, docTemplate, jsonItem, dependencies);
		expect(testObject.widget).toBeDefined();
		expect(testObject.widget.fields[0].dependsOn).not.toBeDefined();
		expect(testObject.container).toBeDefined();
	});

	it("GetFormWidgetAndContainer should return an object with widget and container when property.type is array", function(){
		var dependencies = {};
		var fieldsNames = ["anyProperty"];
		var docTemplate = {
			"$schema": "http://json-schema.org/draft-04/schema#",
			"description": "A form widget",
			"properties": {
				"anyProperty": {
					"type": ["object"],
					"description": "anyDescription"
				}
			},
			"required": ["anyProperty"],
			"title": "form.html",
			"type": "object"
		};
		var jsonItem = {"anyProperty": []};
		var testObject = DevToolsForm.getFormWidgetAndContainer(fieldsNames, docTemplate, jsonItem, dependencies);
		expect(testObject.widget.fields[0].template).toBe("field/textarea.html");
		expect(testObject.container).toBeDefined();
		expect(testObject.widget).toBeDefined();
	});

	it("GetFormWidgetAndContainer should return an object with widget and container using defaultValues (testing checkbox and not defined properties)", function(){
		var dependencies = {};
		var fieldsNames = ["anyProperty", "secondProperty", "template"];
		var docTemplate = {
			"$schema": "http://json-schema.org/draft-04/schema#",
			"description": "A form widget",
			"properties": {
				"anyProperty": {
					"type": "anyType",
					"description": "anyDescription",
					"default": "defaultValue"
				},
				"secondProperty": {
					"type": "array",
					"description": "array of anything",
					"default": []
				},
				"template": {
					"type": "string",
					"description": "the styles of the screen",
					"default": "field/checkbox.html"
				}
			},
			"required": ["anyProperty"],
			"title": "form.html",
			"type": "object"
		};
		var jsonItem = {"isVisible":true,"secondProperty": ['value']};
		var testObject = DevToolsForm.getFormWidgetAndContainer(fieldsNames, docTemplate, jsonItem, dependencies);
		expect(testObject.widget.fields[0].defaultValue).toBe("defaultValue");
		expect(testObject.widget.fields[2].defaultValue).toBe("field/checkbox.html");
		expect(testObject.widget).toBeDefined();
		expect(testObject.container).toBeDefined();
	});

	it("GetFormWidgetAndContainer should return an object with widget and container when fieldsNames is an empty array", function(){
		var dependencies = {};
		var fieldsNames = [];
		var docTemplate = {
			"$schema": "http://json-schema.org/draft-04/schema#",
			"description": "A form widget",
			"properties": {
				"anyProperty": {
					"type": "anyType",
					"description": "anyDescription"
				}
			},
			"required": ["anyProperty"],
			"title": "form.html",
			"type": "object"
		};
		var jsonItem = {"anyProperty": "anyValue"};
		var testObject = DevToolsForm.getFormWidgetAndContainer(fieldsNames, docTemplate, jsonItem, dependencies);
		expect(testObject.widget).toBeDefined();
		expect(testObject.widget.fields[0].name).toBe("template");
		expect(testObject.widget.fields.length).toBe(1);
		expect(testObject.container).toBeDefined();
	});

	it("GetFormWidgetAndContainer should return an object with widget and container when jsonItem is an empty object", function(){
		var dependencies = {};
		var fieldsNames = [];
		var docTemplate = {
			"$schema": "http://json-schema.org/draft-04/schema#",
			"description": "A form widget",
			"properties": {
				"anyProperty": {
					"type": "anyType",
					"description": "anyDescription"
				}
			},
			"required": ["anyProperty"],
			"title": "form.html",
			"type": "object"
		};
		var jsonItem = {};
		var testObject = DevToolsForm.getFormWidgetAndContainer(fieldsNames, docTemplate, jsonItem, dependencies);
		expect(testObject.widget.fields[0].name).toBe("template");
		expect(testObject.widget).toBeDefined();
		expect(testObject.container).toBeDefined();
	});

	it("GetFormWidgetAndContainer should return an object with a widget property containing fields without field.description when not defined at docTemplate", function(){
		var dependencies = {};
		var fieldsNames = ["anyProperty", "otherProperty"];
		var docTemplate = {
			"$schema": "http://json-schema.org/draft-04/schema#",
			"description": "A form widget",
			"properties": {
				"anyProperty": {
					"type": "boolean",
					"description": "anyDescription"
				}
			},
			"required": ["anyProperty"],
			"title": "form.html",
			"type": "object"
		};
		var jsonItem = {
			"anyProperty": "anyValue",
			"otherProperty": "value"
		};
		var testObject = DevToolsForm.getFormWidgetAndContainer(fieldsNames, docTemplate, jsonItem, dependencies);
		expect(testObject.widget.fields[0].name).toBe("anyProperty");
		expect(testObject.widget.fields[1].name).toBe("otherProperty");
	});

	it("GetFormDependencies should return an empty object when docDependencies is undefined and there isn't an option", function(){
		var docDependencies = undefined;
		var options = false;
		expect(JSON.stringify(DevToolsForm.getFormDependencies(docDependencies, options))).toBe(JSON.stringify({}));
	});

	it("GetFormDependencies should return an dependencies object with a property dependsOn constructed by the entry's docDependencies dependencies", function(){
		var docDependencies = {
			"dependency": ["dependency01", "dependency02"]
		};
		var options = {
			"dependency": {
				"dependencies": {
					"dependency01": "value",
					"dependency02": "value"
				}
			}
		};
		var dependsOn = {
			"dependency": {
				"dependency01": "value",
				"dependency02": "value"
			}
		};
		var testObject = DevToolsForm.getFormDependencies(docDependencies, options);
		expect(JSON.stringify(testObject.dependsOn)).toBe(JSON.stringify(dependsOn));
	});

	it("GetFormDependencies should return an dependencies object with an empty property dependsOn when options[docDependencies.property] isn't defined", function(){
		var docDependencies = {
			"dependency": ["dependency01", "dependency02"]
		};
		var options = {
			"dependencies": {
				"dependency01": "value",
				"dependency02": "value"
			}
		};
		var testObject = DevToolsForm.getFormDependencies(docDependencies, options);
		expect(JSON.stringify(testObject.dependsOn)).toBe(JSON.stringify({}));
	});

	it("GetFormDependencies should return an dependencies object with a property dependenciesTree with the tree of dependencies", function(){
		var docDependencies = {
			"dep1": ["value", "otherValue"],
			"dep2": ["value", "anotherValue"],
			"dep3": ["value", "newValue"]
		};
		var options = {
			"dep1": {
				"dependencies": {
					"value": "value"
				}
			},
			"dep2": {
				"dependencies": {
					"otherValue": "otherValue"
				}
			},
			"dep3": {
				"dependencies": {
					"value": "value"
				}
			}
		};
		var preparedDep = {
			"value": ["dep1", "dep3"]
		};
		var testObject = DevToolsForm.getFormDependencies(docDependencies, options);
		expect(JSON.stringify(testObject.dependenciesTree)).toBe(JSON.stringify(preparedDep));
	});

	it("GetFormDependencies should return an dependencies object with an empty property dependsOn when docDependencies or options are undefined", function(){
		var docDependencies = undefined;
		var options = undefined;
		var testObject = DevToolsForm.getFormDependencies(docDependencies, options);
		expect(testObject.dependsOn).not.toBeDefined();
	});

	it("GetActiveFormState should return the state's object with a property value to be an object (truthy) when the entries are valid", function(){
		var dependencies = {};
		var fieldsNames = ["anyProperty"];
		var docTemplate = {
			"$schema": "http://json-schema.org/draft-04/schema#",
			"description": "A form widget",
			"properties": {
				"anyProperty": {
					"type": "anyType",
					"description": "anyDescription"
				}
			},
			"required": ["anyProperty"],
			"title": "form.html",
			"type": "object"
		};
		var jsonItem = {"anyProperty": "anyValue"};
		var formWidget = DevToolsForm.getFormWidgetAndContainer(fieldsNames, docTemplate, jsonItem, dependencies);
		var widgetToForm = new Field();
		widgetToForm.anyProperty = jsonItem.anyProperty;
		var activeItemJson = {
			"item": widgetToForm,
			"text":JSON.stringify(jsonItem)
		};
		formWidget.widget.isValid = function(){
			return true;
		};
		var testObject = DevToolsForm.getActiveFormState(activeItemJson, formWidget.widget);
		expect(!!testObject.value).toEqual(true);
	});

	it("GetActiveFormState should return the state's object with a property value to be an object (truthy) when the entries are valid (fieldValue != defaultValue)", function(){
		var dependencies = {};
		var fieldsNames = ["anyProperty"];
		var docTemplate = {
			"$schema": "http://json-schema.org/draft-04/schema#",
			"description": "A form widget",
			"properties": {
				"anyProperty": {
					"type": "anyType",
					"description": "anyDescription",
					"default": "anything"
				}
			},
			"required": ["anyProperty"],
			"title": "form.html",
			"type": "object"
		};
		var jsonItem = {"anyProperty": "anyValue"};
		var formWidget = DevToolsForm.getFormWidgetAndContainer(fieldsNames, docTemplate, jsonItem, dependencies);
		var widgetToForm = new Field();
		widgetToForm.anyProperty = jsonItem.anyProperty;
		var activeItemJson = {
			"item": widgetToForm,
			"text":JSON.stringify(jsonItem)
		};
		formWidget.widget.isValid = function(){
			return true;
		};
		var testObject = DevToolsForm.getActiveFormState(activeItemJson, formWidget.widget);
		expect(!!testObject.value).toEqual(true);
	});

	it("GetActiveFormState should return the state's object with a property value to be an object (truthy) when the entries are valid (fieldInForm.isVisible is falsy)", function(){
		var dependencies = {};
		var fieldsNames = ["anyProperty"];
		var docTemplate = {
			"$schema": "http://json-schema.org/draft-04/schema#",
			"description": "A form widget",
			"properties": {
				"anyProperty": {
					"type": "anyType",
					"description": "anyDescription",
					"default": "anything"
				}
			},
			"required": ["anyProperty"],
			"title": "form.html",
			"type": "object"
		};
		var jsonItem = {"anyProperty": "anyValue"};
		var formWidget = DevToolsForm.getFormWidgetAndContainer(fieldsNames, docTemplate, jsonItem, dependencies);
		var widgetToForm = new Field();
		widgetToForm.anyProperty = jsonItem.anyProperty;
		var activeItemJson = {
			"item": widgetToForm,
			"text":JSON.stringify(jsonItem)
		};
		formWidget.widget.isValid = function(){
			return true;
		};
		formWidget.widget.fields[0].isVisible = false;
		var testObject = DevToolsForm.getActiveFormState(activeItemJson, formWidget.widget);
		expect(!!testObject.value).toEqual(true);
	});

	it("GetActiveFormState should return the state's object with a property valid equals true when the entries are valid", function(){
		var json = {
			"id": "54294760110983792791095",
			"name": "Dashboard",
			"label": "Dashboard",
			"isVisible": true
		};
		var activeItemJson = {
			"item": new Container(),
			"text":JSON.stringify(json)
		};
		var formWidget = new Widget();
		var testObject = DevToolsForm.getActiveFormState(activeItemJson, formWidget);
		expect(testObject.valid).toBe(true);
	});

	it("GetActiveFormState should return the state's object with a property valid equals false when the formWidget object isn't an Widget instance", function(){
		var json = {
			"id": "value",
			"name": "value",
			"isVisible":true
		};
		var activeItemJson = {"text":JSON.stringify(json)};
		var formWidget = new Field();
		var testObject = DevToolsForm.getActiveFormState(activeItemJson, formWidget);
		expect(testObject.valid).toBe(false);
	});

	it("GetActiveFormState should return the state's object with a property valid equals false when there's no text into the ActiveItemJson's object", function(){
		var activeItemJson = {};
		var formWidget = new Field();
		var testObject = DevToolsForm.getActiveFormState(activeItemJson, formWidget);
		expect(testObject.valid).toBe(false);
	});

	it("GetActiveFormState should return the state's object with a property valid equals false when there's no formWidget object being passed", function(){
		var json = {
			"id": "value",
			"name": "value",
			"isVisible":true
		};
		var activeItemJson = {"text":JSON.stringify(json)};
		var testObject = DevToolsForm.getActiveFormState(activeItemJson, undefined);
		expect(testObject.valid).toBe(false);
	});

	it("GetFormDocTemplate of an already defined template should return the json template (schema)" , function(){
		var template = "field/button.html";
		var inOptionsFolder = true;
		var url = "../../dyman/docs/dyman/options/"+template;
		data.url = url;
		$httpBackend.expectGET("/json_test/test/"+url+".json").respond(data);
		var testObject = DevToolsForm.getFormDocTemplate(template, inOptionsFolder);
		testObject.then(function(response){
			expect(response.data).toBe("value");
		});
		$httpBackend.flush();
	});

	it("GetFormDocTemplate of an existing but not defined template should return the default template (schema)" , function(){
		var template = "field/something.html";
		var inOptionsFolder = false;
		var url = "../../dyman/docs/dyman/"+template;
		data.url = url;
		templateManager.getDocsUrl = function(path){
			return path;
		};
		DevToolsService.requestMetadataDeveloper = function(path){
			var deferred = $q.defer();
			deferred.reject(data);
			return deferred.promise;
		};
		var testObject = DevToolsForm.getFormDocTemplate(template, inOptionsFolder);
		$scope.$apply();
		testObject.catch(function(data){
			expect(data.url).toBe(url);
		});
	});

	it("GetFormDocTemplate of an existing but not defined template should return false" , function(){
		var template = "field/anything.html";
		var inOptionsFolder = true;
		var url = "../../dyman/docs/dyman/options/" + template;
		data.url = url;
		templateManager.getDocsUrl = function(path){
			return path;
		};
		DevToolsService.requestMetadataDeveloper = function(path){
			var deferred = $q.defer();
			deferred.reject(data);
			return deferred.promise;
		};
		var testObject = DevToolsForm.getFormDocTemplate(template, inOptionsFolder);
		$scope.$apply();
		testObject.catch(function(data){
			expect(data.url).toBe(url);
		});
	});

	it("ValueFunctionObjectField is initialized. - field.value() should return what's passed by field.value(parameter)", function(){
		var dependencies = {};
		var fieldsNames = ["anyProperty"];
		var docTemplate = {
			"$schema": "http://json-schema.org/draft-04/schema#",
			"description": "A form widget",
			"properties": {
				"anyProperty": {
					"type": ["object"],
					"description": "anyDescription"
				}
			},
			"required": ["anyProperty"],
			"title": "form.html",
			"type": "object"
		};
		var jsonItem = {"anyProperty": []};
		var testObject = DevToolsForm.getFormWidgetAndContainer(fieldsNames, docTemplate, jsonItem, dependencies);
		docTemplate.properties.type = ["array"];
		var test = testObject.widget.fields[0];
		var toBeResult = JSON.stringify({"property": "value"});
		expect(test.value()).toBe('');
		test.value({"property": "value"});
		expect(JSON.stringify(test.value())).toBe(toBeResult);
		test.value(toBeResult);
		expect(JSON.stringify(test.value())).toBe(toBeResult);
	});

});