/* global Util, UtilDevTools, ShowInDevTools, Configuration */
(function(){
    function MenuTreeService(ElementToMetadataService, templateManager){
        var self = this;
        var lastItem;
        var treeItems = [];
        var MENU_PROPERTY = {
            "type": "menus",
            "parent": "groupMenu"
        };
        var menuUnloadedItems;

        this.setUnloadedItems = function(unloadedItems){
            menuUnloadedItems = unloadedItems;
        };
        this.getUnloadedItems = function(){
            return menuUnloadedItems;
        };

        this.setTreeInWindow = function(devToolsWindow){
            var menu = {};
            menu.groupMenu = Util.clone(templateManager.containers.zeedhi_project.groupMenu);
            menu.__indexPath = templateManager.project.__indexPath;
            menu.label = "groupMenu";
            devToolsWindow.groupMenu = menu;
        };

        this.setLastItem = function(item){
            lastItem = item;
        };

        this.getLastItem = function(){
            return lastItem;
        };

        var setTreeItems = function(items){
            treeItems = items;
        };

        this.getTreeItems = function(){
            return treeItems;
        };

        var setContainersInGroupMenu = function(groupMenu){
            groupMenu.containers = groupMenu.containers || templateManager.containers;
            groupMenu.containers.__metadata = Util.clone(groupMenu.containers.zeedhi_project);
        };

        this.getItemMetadata = function(item){
            var groupMenu = UtilDevTools.getDevToolsWindow().groupMenu;
            setContainersInGroupMenu(groupMenu);
            return ElementToMetadataService.getElementToMetadata(item, groupMenu.containers);
        };

        this.openTree = function(pathToOpen, lastItem){
            if(pathToOpen.length >= 2 && lastItem){
                var type = pathToOpen.shift();
                var index = pathToOpen.shift();
                lastItem._opened = true;
                var item = lastItem[type] || lastItem;
                if(item[index]){
                    lastItem = item[index];
                    return self.openTree(pathToOpen, lastItem);
                }
            }
            return lastItem;
        };

        var setParentInTree = function(parent, groupMenuElements){
            var element = parent[MENU_PROPERTY.type] || parent[MENU_PROPERTY.parent];
            var parentProperty = groupMenuElements ? 'parentMenu' : 'parent';
            if(Array.isArray(element)){
                element.forEach(function(subElement){
                    subElement[parentProperty] = parent;
                    setParentInTree(subElement);
                });
            }
            return parent;
        };

        this.loadTree = function(devToolsWindow){
            devToolsWindow = devToolsWindow || UtilDevTools.getDevToolsWindow();
            var hasDevToolsWindowAndGroupMenu = devToolsWindow && devToolsWindow.groupMenu;
            if (hasDevToolsWindowAndGroupMenu){
                ShowInDevTools.setTree(devToolsWindow.groupMenu);
                var newGroupMenu = setParentInTree(devToolsWindow.groupMenu, true);
                setTreeItems([newGroupMenu]);
            }
        };
        this.updateTree = self.loadTree;

        this.getDefaultTreeItem = function(){
            return window.groupMenu;
        };

        this.getTotalUnloadedItems = function(groupMenu){
            var PROJECT = 'project';
            var GROUPMENU_MAP = {
                project: ["groupMenu"],
                groupMenu: ["menus"],
                menus: ["menus"]
            };
            var hasSomeUnloadedItems = !groupMenu._element;
            var getTotalItems = function(item, type, initValue){
                var total = initValue;
                var itemProperties = GROUPMENU_MAP[type];
                if(itemProperties){
                    total += itemProperties.reduce(function(total, property){
                        var children = item[property];
                        if(children && children.length){
                            total += children.reduce(function(total, child){
                                if(child && child.showInDevTools){
                                    total++;
                                    hasSomeUnloadedItems = hasSomeUnloadedItems || !child._element;
                                    return total + getTotalItems(child, property, 0);
                                } else {
                                    return total;
                                }
                            }, 0);
                        }
                        return total;
                    }, 0);
                }
                return total;
            };
            var totalUnloadedItems = getTotalItems(groupMenu, PROJECT, 1);
            var isOpener = window.opener;
            totalUnloadedItems = (hasSomeUnloadedItems || isOpener) ? totalUnloadedItems : 0;
            return totalUnloadedItems;
        };
    }

    Configuration(function(DeveloperTreeService, ElementToMetadataService, templateManager){
        var instance = new MenuTreeService(ElementToMetadataService, templateManager);
        DeveloperTreeService.insertTreeMode('menu', instance);
    });
})();
